sentences = ["This is not a test.",
             "There is no flowery dialog, and time is not wasted.",
             "She did not promise to help him.",
             "The King of France is not bald.",
             "It is not so much a work of entertainment as it is unique study.",
             "Mary did not complete the program but Nancy wrote the report.",
             "Not an accomplished dancer, he moved rather clumsily.",
             "Not all participants liked this game.",
             "I do not think he is coming.",
             "Mary did not give the solution to Paul.",
             "She claimed that Donald had not offered bribes to any official.",
             "Not for the first time, he was surprised by this player.",
             "I would never do it even if I can.",
             "A decision is not expected until June.",
             "We do not like washing dishes which lead to the decision of buying a dishwasher."
             ]

from nltk.parse.corenlp import CoreNLPServer
from nltk.parse.corenlp import CoreNLPParser
set_negatives = set(['no','not','never'])
# https://nlp.stanford.edu/software/stanford-parser-full-2018-10-17.zip

def is_negative_tree(tree):
    lower_leaves = [x.lower() for x in tree.leaves()]
    if len(set_negatives.intersection(lower_leaves)) > 0 and tree.label() not in ['RB','DT']:
        return True
    else:
        return False

def deepest_negative_subtree(tree):
    negative_subtrees = []
    negative_subtrees_height = []
    for s in tree.subtrees():
        if is_negative_tree(s) and len(s.leaves())>1:
            negative_subtrees.append(s)
            negative_subtrees_height.append(s.height())
    if len(negative_subtrees) == 0:
        return tree
    else:
        for t in negative_subtrees:
            if t.height() == min(negative_subtrees_height):
                return t
            
def negative_scope_pos(pos_list):
    scope_pos_list = []
    scope = False
    for pos_word in pos_list:
        # Fin du scope
        if pos_word[1] in ['WDT','RB',','] and scope==True and len(scope_pos_list)>0:
            scope = False
        # Intérieur du scope
        if scope==True:
            scope_pos_list.append(pos_word)
        # Début du scope
        if pos_word[0].lower() in set_negatives:
            scope = True
    return scope_pos_list
            

def convert_negated_words(sentence):
    # Pour évaluer cette tâche, nous allons utiliser cette fonction pour tester la portée de la négation
    # d'autres phrases que celles dans la liste plus haut (sentences).
    # SVP ne pas changer la signautre de la fonction.
    #
    # Pour déterminer la portée d'une négation, utilisez la structure d'un arbre syntaxique (constituants ou dépendances),
    # les symboles des consituants ou des dépendance, et les part-of-speech des mots (POS).
    #
    # Vous pouvez utiliser la libairie de votre choix pour faire l'analyse syntaxique des phrases.
    # Et vous DEVEZ utiliser un analyseur syntaxique.
    #
    # Vous pouvez ajouter autant de fonctions que vous le souhaitez.
    # Mais vous ne pouvez pas utiliser de code disponible sur le Web pour déterminer la portée de négations.
    #
    # Nos solutions pour les 15 phrases sont dans le fichier /data/phrases-references-t2.txt
    #
    # Mettre votre code ici. Vous pouvez effacer ces commentaires.
    #
    
    # Analyser la phrase
    parse = next(parser.raw_parse(sentence))
    
    # Identifier les parties négatives dans les sous arbres
    dns_pos_list = []
    for st in parse.subtrees():
        if is_negative_tree(st):
            # Plus petit sous-arbre négatif
            current_dns = deepest_negative_subtree(st)
            print("sous-arbre courant: "+str(current_dns))
            # Section affectée par la négation
            dns_pos_list.append(negative_scope_pos(current_dns.pos()))
    
    # Sélectionner les sections négatives uniques (plusieurs sous-arbres génèrent la même section)
    unique_dns_pos_list = [x for i, x in enumerate(dns_pos_list) if i == dns_pos_list.index(x)]
    
    # Produire les remplacements à effecture
    converted_sentence = sentence
    for dns_pos in unique_dns_pos_list:
        to_replace = " ".join(x[0] for x in dns_pos)
        replacement = " ".join("NOT_"+x[0] for x in dns_pos)
        converted_sentence = converted_sentence.replace(to_replace,replacement)
            
    return converted_sentence.strip()


if __name__ == '__main__':
    #server = CoreNLPServer("/home/francois/stanford-corenlp-full-2018-10-05/stanford-corenlp-3.9.2.jar",
    #                       "/home/francois/stanford-corenlp-full-2018-10-05/stanford-english-corenlp-2018-10-05-models.jar")
    #server.start()
    #parser = CoreNLPParser()
    output_file = open("/home/francois/nlp_a2019_tp2/nlp_a2019_tp2/output_negative.txt","w")
    for sent in sentences:
        print("\nS:", sent)
        output_file.write("S: "+sent)
        # Modifications pour ressembler au fichier du prof qui a des espaces devant les ponctuations
        converted = convert_negated_words(sent).replace("."," .").replace(","," ,")
        print("N:", converted)
        output_file.write("\nN: "+converted+"\n\n")
    output_file.close()
    #server.stop()